% File src/library/base/man/apply.Rd
% Part of the R package, https://www.R-project.org
% Copyright (C) 1995-2015 R Core Team
% Distributed under GPL 2 or later

\name{apply}
\alias{apply}
\title{Apply Functions Over Array Margins}
\description{
  関数を配列または行列のマージンに適用して得られる値の
ベクトル，配列またはリストを返す．
}
\usage{
apply(X, MARGIN, FUN, \dots)
}
\arguments{
  \item{X}{配列，行列を含む．}
  \item{MARGIN}{関数が適用されるべき添字を与えるベクトル．
    例えば行列に対しては \code{1} は行，
    \code{2} は列，\code{c(1, 2)} は行と列を指示する．
    \code{X} が名前付きの次元を持てば，これは次元名を選択する文字列ベクトル
でも良い．}
  \item{FUN}{適用される関数：\sQuote{詳細}を見よ．
    \code{+}, \code{\%*\%} 等の関数の場合関数名は逆引用化もしくは
    引用化されなければならない．}
  \item{\dots}{\code{FUN} への追加引数．}
}
\details{
  もし \code{X} が配列ではないが(データフレームのような)ヌルでない \code{\link{dim}} 値を
持つクラスのオブジェクトなら，\code{apply} はそれを2次元なら \code{as.matrix} を使い
行列に，または \code{as.array} を使い配列に強制変換する．

  \code{FUN} は \code{\link{match.fun}} の呼び出しで探索され，
典型的には関数かシンボル(バックコートで引用化された名前)もしくは
 \code{apply} への呼び出し環境から探索される関数を指定する文字列
である．

\code{\dots} 中の引数は他の引数のどれかと同じ名前を持つことはできず，
 \code{MARGIN} や \code{FUN} への部分マッチングを避けるように注意が
必要である．一般的用途向けのコード中では，もし \code{\dots} に引数を渡すならば，
最初の三つの引数には名前をつけ，仮に
引数 \code{X}, \code{MARGIN} そして \code{FUN} の名前が \code{\dots} を
通じて渡されたらそれと分かるようなエラーが起こるようにすることは良い作法である．.
}
\value{
もし \code{FUN} への各呼び出しが長さ \code{n} のベクトルを返すならば，
 \code{n > 1} であれば \code{apply} は次元 \code{c(n, dim(X)[MARGIN])} の配列を返す．
もし \code{n} が \code{1} に等しければ，\code{apply} は \code{MARGIN} が長さ1ならば
ベクトルを，さもなければ次元 \code{dim(X)[MARGIN]} の配列を返す．
もし \code{n} が \code{0} ならば，結果は長さが 0 の必ずしも正確とは言えない
次元を持つ．

もし \code{FUN} への呼び出しが異なった長さのベクトルを返すのならば，
 \code{apply} はリストを返し，長さは \code{prod(dim(X)[MARGIN])} であるが
もしこれが 1 より大きな長さを持てば \code{dim} は \code{MARGIN} に設定される．

すべてのケースで結果は次元の設定前に \code{\link{as.vector}} で
基本ベクトル型の一つに変換されるので，(例えば)因子は文字列配列に
強制変換されるであろう．
}
\references{
  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
  \emph{The New S Language}.
  Wadsworth & Brooks/Cole.
}
\seealso{
  \code{\link{lapply}} とその同類 \code{\link{simplify2array}}；
  \code{\link{tapply}}，そして簡便化関数の
  \code{\link{sweep}} と \code{\link{aggregate}}．
}
\examples{
## 行列の列和と行和を計算：
x <- cbind(x1 = 3, x2 = c(4:1, 2:5))
dimnames(x)[[1]] <- letters[1:8]
apply(x, 2, mean, trim = .2)
col.sums <- apply(x, 2, sum)
row.sums <- apply(x, 1, sum)
rbind(cbind(x, Rtot = row.sums), Ctot = c(col.sums, sum(col.sums)))

stopifnot( apply(x, 2, is.vector))

## 行列の列をソート
apply(x, 2, sort)

## 名前付き次元名を保存
names(dimnames(x)) <- c("row", "col")
x3 <- array(x, dim = c(dim(x),3),
	    dimnames = c(dimnames(x), list(C = paste0("cop.",1:3))))
identical(x,  apply( x,  2,  identity))
identical(x3, apply(x3, 2:3, identity))
\dontshow{
xN <- x; dimnames(xN) <- list(row=NULL, col=NULL)
x2 <- x; names(dimnames(x2)) <- NULL
fXY <- function(u) c(X=u[1], Y=u[2])
ax1 <- apply(x, 1, fXY)
ax2 <- apply(x2,1, fXY)
stopifnot(identical(dimnames(ax1), list(col=c("X.x1", "Y.x2"), row=letters[1:8])),
          identical(dimnames(ax2), unname(dimnames(ax1))),
          identical( x, apply( x, 2, identity)),
          identical(xN, apply(xN, 2, identity)),
	  identical(dimnames(x),
		    dimnames(apply(x, 2, format))),
          identical(x3, apply(x3, 2:3, identity)),
	  identical(dimnames(apply(x3, 2:1, identity)),
		    dimnames(x3)[3:1]))
rm(xN, x2, fXY, ax1, ax2)
}
##- 追加引数を持つ関数：
cave <- function(x, c1, c2) c(mean(x[c1]), mean(x[c2]))
apply(x, 1, cave,  c1 = "x1", c2 = c("x1","x2"))

ma <- matrix(c(1:4, 1, 6:8), nrow = 2)
ma
apply(ma, 1, table)  #--> a list of length 2
apply(ma, 1, stats::quantile) # 行名を持つ 5 x n 行列

stopifnot(dim(ma) == dim(apply(ma, 1:2, sum)))

## 各呼び出し毎に異なった長さを持つ例
z <- array(1:24, dim = 2:4)
zseq <- apply(z, 1:2, function(x) seq_len(max(x)))
zseq         ## 2 x 3 行列
typeof(zseq) ## リスト
dim(zseq) ## 2 3
zseq[1,]
apply(z, 3, function(x) seq_len(max(x)))
# 次元属性を持たないリスト
}
\keyword{iteration}
\keyword{array}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/apply.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright (C) 1995-2015 R Core Team
%% Distributed under GPL 2 or later
%
%\name{apply}
%\alias{apply}
%\title{Apply Functions Over Array Margins}
%\description{
%  Returns a vector or array or list of values obtained by applying a
%  function to margins of an array or matrix.
%}
%\usage{
%apply(X, MARGIN, FUN, \dots)
%}
%\arguments{
%  \item{X}{an array, including a matrix.}
%  \item{MARGIN}{a vector giving the subscripts which the function will
%    be applied over.  E.g., for a matrix \code{1} indicates rows,
%    \code{2} indicates columns, \code{c(1, 2)} indicates rows and
%    columns. Where \code{X} has named dimnames, it can be a character
%    vector selecting dimension names.}
%  \item{FUN}{the function to be applied: see \sQuote{Details}.
%    In the case of functions like \code{+}, \code{\%*\%}, etc., the
%    function name must be backquoted or quoted.}
%  \item{\dots}{optional arguments to \code{FUN}.}
%}
%\details{
%  If \code{X} is not an array but an object of a class with a non-null
%  \code{\link{dim}} value (such as a data frame), \code{apply} attempts
%  to coerce it to an array via \code{as.matrix} if it is two-dimensional
%  (e.g., a data frame) or via \code{as.array}.
%
%  \code{FUN} is found by a call to \code{\link{match.fun}} and typically
%  is either a function or a symbol (e.g., a backquoted name) or a
%  character string specifying a function to be searched for from the
%  environment of the call to \code{apply}.
%
%  Arguments in \code{\dots} cannot have the same name as any of the
%  other arguments, and care may be needed to avoid partial matching to
%  \code{MARGIN} or \code{FUN}.  In general-purpose code it is good
%  practice to name the first three arguments if \code{\dots} is passed
%  through: this both avoids partial matching to \code{MARGIN}
%  or \code{FUN} and ensures that a sensible error message is given if
%  arguments named \code{X}, \code{MARGIN} or \code{FUN} are passed
%  through \code{\dots}.
%}
%\value{
%  If each call to \code{FUN} returns a vector of length \code{n}, then
%  \code{apply} returns an array of dimension \code{c(n, dim(X)[MARGIN])}
%  if \code{n > 1}.  If \code{n} equals \code{1}, \code{apply} returns a
%  vector if \code{MARGIN} has length 1 and an array of dimension
%  \code{dim(X)[MARGIN]} otherwise.
%  If \code{n} is \code{0}, the result has length 0 but not necessarily
%  the \sQuote{correct} dimension.
%
%  If the calls to \code{FUN} return vectors of different lengths,
%  \code{apply} returns a list of length \code{prod(dim(X)[MARGIN])} with
%  \code{dim} set to \code{MARGIN} if this has length greater than one.
%
%  In all cases the result is coerced by \code{\link{as.vector}} to one
%  of the basic vector types before the dimensions are set, so that (for
%  example) factor results will be coerced to a character array.
%}
%\references{
%  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
%  \emph{The New S Language}.
%  Wadsworth & Brooks/Cole.
%}
%\seealso{
%  \code{\link{lapply}} and there, \code{\link{simplify2array}};
%  \code{\link{tapply}}, and convenience functions
%  \code{\link{sweep}} and \code{\link{aggregate}}.
%}
%\examples{
%## Compute row and column sums for a matrix:
%x <- cbind(x1 = 3, x2 = c(4:1, 2:5))
%dimnames(x)[[1]] <- letters[1:8]
%apply(x, 2, mean, trim = .2)
%col.sums <- apply(x, 2, sum)
%row.sums <- apply(x, 1, sum)
%rbind(cbind(x, Rtot = row.sums), Ctot = c(col.sums, sum(col.sums)))
%
%stopifnot( apply(x, 2, is.vector))
%
%## Sort the columns of a matrix
%apply(x, 2, sort)
%
%## keeping named dimnames
%names(dimnames(x)) <- c("row", "col")
%x3 <- array(x, dim = c(dim(x),3),
%	    dimnames = c(dimnames(x), list(C = paste0("cop.",1:3))))
%identical(x,  apply( x,  2,  identity))
%identical(x3, apply(x3, 2:3, identity))
%\dontshow{
%xN <- x; dimnames(xN) <- list(row=NULL, col=NULL)
%x2 <- x; names(dimnames(x2)) <- NULL
%fXY <- function(u) c(X=u[1], Y=u[2])
%ax1 <- apply(x, 1, fXY)
%ax2 <- apply(x2,1, fXY)
%stopifnot(identical(dimnames(ax1), list(col=c("X.x1", "Y.x2"), row=letters[1:8])),
%          identical(dimnames(ax2), unname(dimnames(ax1))),
%          identical( x, apply( x, 2, identity)),
%          identical(xN, apply(xN, 2, identity)),
%	  identical(dimnames(x),
%		    dimnames(apply(x, 2, format))),
%          identical(x3, apply(x3, 2:3, identity)),
%	  identical(dimnames(apply(x3, 2:1, identity)),
%		    dimnames(x3)[3:1]))
%rm(xN, x2, fXY, ax1, ax2)
%}
%##- function with extra args:
%cave <- function(x, c1, c2) c(mean(x[c1]), mean(x[c2]))
%apply(x, 1, cave,  c1 = "x1", c2 = c("x1","x2"))
%
%ma <- matrix(c(1:4, 1, 6:8), nrow = 2)
%ma
%apply(ma, 1, table)  #--> a list of length 2
%apply(ma, 1, stats::quantile) # 5 x n matrix with rownames
%
%stopifnot(dim(ma) == dim(apply(ma, 1:2, sum)))
%
%## Example with different lengths for each call
%z <- array(1:24, dim = 2:4)
%zseq <- apply(z, 1:2, function(x) seq_len(max(x)))
%zseq         ## a 2 x 3 matrix
%typeof(zseq) ## list
%dim(zseq) ## 2 3
%zseq[1,]
%apply(z, 3, function(x) seq_len(max(x)))
%# a list without a dim attribute
%}
%\keyword{iteration}
%\keyword{array}
%
\encoding{UTF8}
