% File src/library/base/man/bindenv.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2014 R Core Team
% Distributed under GPL 2 or later

\name{bindenv}
\title{Binding and Environment Locking, Active Bindings}
\usage{
lockEnvironment(env, bindings = FALSE)
environmentIsLocked(env)
lockBinding(sym, env)
unlockBinding(sym, env)
bindingIsLocked(sym, env)

makeActiveBinding(sym, fun, env)
bindingIsActive(sym, env)
}
\alias{bindenv}
\alias{lockEnvironment}
\alias{environmentIsLocked}
\alias{lockBinding}
\alias{unlockBinding}
\alias{makeActiveBinding}
\alias{bindingIsLocked}
\alias{bindingIsActive}
\arguments{
  \item{env}{環境．}
  \item{bindings}{結合がロックされるべきかどうかを指示する論理値．}
  \item{sym}{名前オブジェクトまたは文字列．}
  \item{fun}{引数がゼロか一個の関数．}
}
\description{
  これらの関数は環境の調整と環境中の結合のための実験的インタフェイスである．
  環境と並んで個々の結合をロックすることや，
  変数を関数にリンクすることを可能にする．
}
\details{
  関数 \code{lockEnvironment} はその環境引数をロックする．環境は普通の
  環境(基礎でない)でなければならない．(基礎環境と名前空間のロックは
  後でサポートされるかもしれない．)  環境のロックは環境からの変数の結合を
  加えたり取り除いたりすることを妨げる．
  変数の値の変更は結合がロックされていない限り依然として可能である．
  名前空間を持つパッケージの名前空間環境は読み込まれるときにロックされる．

  \code{lockBinding} は特定の環境中の個々の結合をロックする．
  ロックされた結合の値は変更できない．
  環境がロックされていない限り，ロックされた結合を環境から取り除くことは可能である．

  \code{makeActiveBinding} は \code{fun} を環境 \code{env} 中に
  インストールし，引数なしの \code{fun} の \code{sym} 呼び出しの値を得たり，
  \code{sym} 呼び出しに割り当てられる値である引数が一つの \code{fun} を割り当て
  たりする．これは\R 変数にリンクされた C 変数やデータベースにリンクされた
  変数のような物事の実装を可能にする．そして \code{\link{setRefClass}} の
  実装に使うことが出来る．これはまた幾つかのシステムの大局の
  スレッドに対して安全なバージョンを作るのに有用かもしれない．
}
\value{
  \code{bindingIsLocked} と \code{environmentIsLocked} は長さ1の
  論理値を返す．他の関数は \code{NULL} を不可視で返す．
}
\examples{
# 環境をロックする
e <- new.env()
assign("x", 1, envir = e)
get("x", envir = e)
lockEnvironment(e)
get("x", envir = e)
assign("x", 2, envir = e)
try(assign("y", 2, envir = e)) # エラー

# 結合をロックする
e <- new.env()
assign("x", 1, envir = e)
get("x", envir = e)
lockBinding("x", e)
try(assign("x", 2, envir = e)) # エラー
unlockBinding("x", e)
assign("x", 2, envir = e)
get("x", envir = e)

# アクティブな結合
f <- local( {
    x <- 1
    function(v) {
       if (missing(v))
           cat("get\n")
       else {
           cat("set\n")
           x <<- v
       }
       x
    }
})
makeActiveBinding("fred", f, .GlobalEnv)
bindingIsActive("fred", .GlobalEnv)
fred
fred <- 2
fred
}
\keyword{utilities}
\author{Luke Tierney}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/bindenv.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2014 R Core Team
%% Distributed under GPL 2 or later
%
%\name{bindenv}
%\title{Binding and Environment Locking, Active Bindings}
%\usage{
%lockEnvironment(env, bindings = FALSE)
%environmentIsLocked(env)
%lockBinding(sym, env)
%unlockBinding(sym, env)
%bindingIsLocked(sym, env)
%
%makeActiveBinding(sym, fun, env)
%bindingIsActive(sym, env)
%}
%\alias{bindenv}
%\alias{lockEnvironment}
%\alias{environmentIsLocked}
%\alias{lockBinding}
%\alias{unlockBinding}
%\alias{makeActiveBinding}
%\alias{bindingIsLocked}
%\alias{bindingIsActive}
%\arguments{
%  \item{env}{an environment.}
%  \item{bindings}{logical specifying whether bindings should be locked.}
%  \item{sym}{a name object or character string.}
%  \item{fun}{a function taking zero or one arguments.}
%}
%\description{
%  These functions represent an experimental interface for adjustments
%  to environments and bindings within environments.  They allow for
%  locking environments as well as individual bindings, and for linking
%  a variable to a function.
%}
%\details{
%  The function \code{lockEnvironment} locks its environment argument,
%  which must be a normal environment (not base).  (Locking the base
%  environment and namespace may be supported later.)  Locking the
%  environment prevents adding or removing variable bindings from the
%  environment.  Changing the value of a variable is still possible unless
%  the binding has been locked.  The namespace environments of packages
%  with namespaces are locked when loaded.
%
%  \code{lockBinding} locks individual bindings in the specified
%  environment.  The value of a locked binding cannot be changed.  Locked
%  bindings may be removed from an environment unless the environment is
%  locked.
%
%  \code{makeActiveBinding} installs \code{fun} in environment \code{env}
%  so that getting the value of \code{sym} calls \code{fun} with no
%  arguments, and assigning to \code{sym} calls \code{fun} with one
%  argument, the value to be assigned.  This allows the implementation of
%  things like C variables linked to \R variables and variables linked to
%  databases, and is used to implement \code{\link{setRefClass}}.  It may
%  also be useful for making thread-safe versions of some system globals.
%}
%\value{
%  The \code{bindingIsLocked} and \code{environmentIsLocked} return a
%  length-one logical vector.  The remaining functions return
%  \code{NULL}, invisibly.
%}
%\examples{
%# locking environments
%e <- new.env()
%assign("x", 1, envir = e)
%get("x", envir = e)
%lockEnvironment(e)
%get("x", envir = e)
%assign("x", 2, envir = e)
%try(assign("y", 2, envir = e)) # error
%
%# locking bindings
%e <- new.env()
%assign("x", 1, envir = e)
%get("x", envir = e)
%lockBinding("x", e)
%try(assign("x", 2, envir = e)) # error
%unlockBinding("x", e)
%assign("x", 2, envir = e)
%get("x", envir = e)
%
%# active bindings
%f <- local( {
%    x <- 1
%    function(v) {
%       if (missing(v))
%           cat("get\n")
%       else {
%           cat("set\n")
%           x <<- v
%       }
%       x
%    }
%})
%makeActiveBinding("fred", f, .GlobalEnv)
%bindingIsActive("fred", .GlobalEnv)
%fred
%fred <- 2
%fred
%}
%\keyword{utilities}
%\author{Luke Tierney}
%
\encoding{UTF8}
