% File src/library/base/man/findInterval.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2009 R Core Team
% Distributed under GPL 2 or later

\name{findInterval}
\alias{findInterval}
\title{Find Interval Numbers or Indices}
\usage{
findInterval(x, vec, rightmost.closed = FALSE, all.inside = FALSE)
}
\arguments{
  \item{x}{数値．}
  \item{vec}{数値．例えば長さ \code{N} で非減少順にソートされている．}
  \item{rightmost.closed}{論理値．もし真なら最右端区間
    \code{vec[N-1] .. vec[N]} は閉じているとされる，下を見よ．}
  \item{all.inside}{論理値．もし真なら返される添字は
    \code{1,\dots,N-1} に変換される．つまり \code{0} は\code{1} へ
    そして \code{N} は \code{N-1} へマップされる．}
}
\description{
  非減少な分割点のベクトルを \code{vec} にあたえた時，
  \code{x} の各点を含む区間を見つける；つまり，もし
  \code{i <- findInterval(x,v)} なら \code{x} 中の各添字 \code{j} 
  \eqn{v_{i_j} \le x_j < v_{i_j + 1}}{v[i[j]] \le x[j] < v[i[j] + 1]}
  ここで \eqn{v_0 := -\infty}{v[0] := - Inf},
  \eqn{v_{N+1} := +\infty}{v[N+1] := + Inf}, そして \code{N <- length(v)}．
  二つの境界では返される添字はオプション \code{rightmost.closed} と \code{all.inside}
  に依存して 1 だけ異なるかもしれない．
}
\details{
  関数 \code{findInterval} はベクトル \code{x} の 別のベクトル
  \code{vec} 中での添字を見つける，ここで後者は非減少でなければならない．
  これが自明である場合 \code{apply( outer(x, vec, ">="), 1, sum)} に同値である．
  実際には区間アルゴリズムは計算量が
  \eqn{O(n \log N)}{O(n * log(N))} である区間探索を行う，
  ここで \code{n <- length(x)} (そして \code{N <- length(vec)}) である．
  (ほとんど)ソートされた \code{x} に対してはこれは更に早く，基本的に \eqn{O(n)} となる．

  これは経験分布関数の計算と同じであり，実際
  \code{findInterval(t, sort(X))} は
  \eqn{n F_n(t; X_1,\dots,X_n)}{n * Fn(t; X[1],..,X[n])} と
  同値である．ここで \eqn{F_n}{Fn} は \eqn{X_1,\dots,X_n}{X[1],..,X[n]} の
  経験分布関数である．

  \code{rightmost.closed = TRUE} の時は，\code{x[j] = vec[N]}
  (\eqn{ = \max vec}{ = max(vec)})に対する結果は最後の区間中のすべての値と
  同様に \code{N - 1} になる．
}
\value{
  \code{0:N} (と \code{NA})に値を取る長さ \code{length(x)} のベクトルで，
  ここで \code{N <- length(vec)}，又は \code{all.inside = TRUE} の時そしてその時だけ
  \code{1:(N-1)} に変換される(同値であるがすべての x 値は区間の内部に変換される)．
  \code{\link{NA}} は \code{x} ｋら伝播し，\code{\link{Inf}} 値が
  \code{x} と \code{vec} の双方に許される．
}
\author{Martin Maechler}
\seealso{\code{findInterval()} の一般化である \code{\link{approx}(*, method = "constant")}，
  (因子 \eqn{n} を除けば) \code{findInterval(.)} と本質的に同値な経験分布関数
  の計算には \code{\link{ecdf}}．
}
\examples{
x <- 2:18
v <- c(5, 10, 15) # 二つのビン [5,10) と [10,15) を作る
cbind(x, findInterval(x, v))

N <- 100
X <- sort(round(stats::rt(N, df = 2), 2))
tt <- c(-100, seq(-2, 2, len = 201), +100)
it <- findInterval(tt, X)
tt[it < 1 | it >= N] # 最初と最後だけが range(x) の外にある
}
\keyword{arith}
\keyword{utilities}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/findInterval.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2009 R Core Team
%% Distributed under GPL 2 or later
%
%\name{findInterval}
%\alias{findInterval}
%\title{Find Interval Numbers or Indices}
%\usage{
%findInterval(x, vec, rightmost.closed = FALSE, all.inside = FALSE)
%}
%\arguments{
%  \item{x}{numeric.}
%  \item{vec}{numeric, sorted (weakly) increasingly, of length \code{N},
%    say.}
%  \item{rightmost.closed}{logical; if true, the rightmost interval,
%    \code{vec[N-1] .. vec[N]} is treated as \emph{closed}, see below.}
%  \item{all.inside}{logical; if true, the returned indices are coerced
%    into \code{1,\dots,N-1}, i.e., \code{0} is mapped to \code{1}
%    and \code{N} to \code{N-1}.}
%}
%\description{
%  Given a vector of non-decreasing breakpoints in \code{vec}, find the
%  interval containing each element of \code{x}; i.e., if
%  \code{i <- findInterval(x,v)}, for each index \code{j} in \code{x}
%  \eqn{v_{i_j} \le x_j < v_{i_j + 1}}{v[i[j]] \le x[j] < v[i[j] + 1]}
%  where \eqn{v_0 := -\infty}{v[0] := - Inf},
%  \eqn{v_{N+1} := +\infty}{v[N+1] := + Inf}, and \code{N <- length(v)}.
%  At the two boundaries, the returned index may differ by 1, depending
%  on the optional arguments \code{rightmost.closed} and \code{all.inside}.
%}
%\details{
%  The function \code{findInterval} finds the index of one vector \code{x} in
%  another, \code{vec}, where the latter must be non-decreasing.  Where
%  this is trivial, equivalent to \code{apply( outer(x, vec, ">="), 1, sum)},
%  as a matter of fact, the internal algorithm uses interval search
%  ensuring \eqn{O(n \log N)}{O(n * log(N))} complexity where
%  \code{n <- length(x)} (and \code{N <- length(vec)}).  For (almost)
%  sorted \code{x}, it will be even faster, basically \eqn{O(n)}.
%
%  This is the same computation as for the empirical distribution
%  function, and indeed, \code{findInterval(t, sort(X))} is
%  \emph{identical} to \eqn{n F_n(t; X_1,\dots,X_n)}{n * Fn(t;
%    X[1],..,X[n])} where \eqn{F_n}{Fn} is the empirical distribution
%  function of \eqn{X_1,\dots,X_n}{X[1],..,X[n]}.
%
%  When \code{rightmost.closed = TRUE}, the result for \code{x[j] = vec[N]}
%  (\eqn{ = \max vec}{ = max(vec)}), is \code{N - 1} as for all other
%  values in the last interval.
%}
%\value{
%  vector of length \code{length(x)} with values in \code{0:N} (and
%  \code{NA}) where \code{N <- length(vec)}, or values coerced to
%  \code{1:(N-1)} if and only if \code{all.inside = TRUE} (equivalently coercing all
%  x values \emph{inside} the intervals).  Note that \code{\link{NA}}s are
%  propagated from \code{x}, and \code{\link{Inf}} values are allowed in
%  both \code{x} and \code{vec}.
%}
%\author{Martin Maechler}
%\seealso{\code{\link{approx}(*, method = "constant")} which is a
%  generalization of \code{findInterval()}, \code{\link{ecdf}} for
%  computing the empirical distribution function which is (up to a factor
%  of \eqn{n}) also basically the same as \code{findInterval(.)}.
%}
%\examples{
%x <- 2:18
%v <- c(5, 10, 15) # create two bins [5,10) and [10,15)
%cbind(x, findInterval(x, v))
%
%N <- 100
%X <- sort(round(stats::rt(N, df = 2), 2))
%tt <- c(-100, seq(-2, 2, len = 201), +100)
%it <- findInterval(tt, X)
%tt[it < 1 | it >= N] # only first and last are outside range(X)
%}
%\keyword{arith}
%\keyword{utilities}
%
\encoding{UTF8}
